/* 
 * Motif Tools Library, Version 3.1
 * $Id: Create.h,v 1.2 2001/04/15 15:29:36 grmcdorman Exp $
 * 
 * Written by David Flanagan.
 * Copyright (c) 1992-2001 by David Flanagan.
 * All Rights Reserved.  See the file COPYRIGHT for details.
 * This is open source software.  See the file LICENSE for details.
 * There is no warranty for this software.  See NO_WARRANTY for details.
 *
 * $Log: Create.h,v $
 * Revision 1.2  2001/04/15 15:29:36  grmcdorman
 *   * XmtCallCallback has two new arguments types: XmtRCallbackUserData, which
 *     is the XmNuserData value, and XmtRCallbackImmediate, which is a value
 *     embedded in the list.
 *
 *   * Two new functions, XmtCreateQueryListChildren and XmtCreateQueryListChild,
 *     are provided. They are like the XmtCreate* functions except that they
 *     return a list of the widgets created (quark and pointer to the widget).
 *
 *   * Major enhancements to XmtInputField:
 *     - The input field now has a cancel() action. This reverts the field's
 *       value to that prior to the start of editing (i.e. the last commit).
 *       No default translation is bound to this action, however.
 *     - There is a callback for the cancel action (XmtNcancelCallback).
 *     - The input field can be set to cancel, commit, or do nothing on focus
 *       out. (Previously, it always did a commit). The default is commit.
 *       (XmtNfocusOutAction)
 *
 *   * XmtLayout:
 *     - It now has a losingFocusCallback (XmNloosingFocusCallback).
 *     - It, and the layout widgets (XmtLayoutBox/XmtLayoutString), now
 *       support background pixmaps.
 *     - It now supports XmNresizePolicy.
 *     - Strings in the layout parser are converted to XmStrings by
 *       XtConvertAndStore, so any custom XmString conversion is used
 *       (as, for example, the Xmt convert, which supports the @[]
 *       embedded items)
 *
 *   * Other:
 *     - Pixmaps are convered using XtConvertAndStore, again to allow
 *       custom converters.
 *
 * Bug Fixes: (for Solaris 2.5.1/Motif 1.2/Sparc system)
 *   * XmtInputField did not work properly if the widget had traversalOn
 *     false.
 *   * XmtLayout's child geometry manager was called by Xt when
 *     a shell child was added to the layout; it did not handle
 *     this case.
 *   * XmtLayout had an XmtWarningMsg call with a missing argument (line 2029).
 *   * Widgets automatically created by XmtLayout parsing have been given
 *     non-blank names. [You may wish to tweak this by adding, for example,
 *     a leading underscore.]
 *   * Under some circumstances, XmtNameToWidget could be called during
 *     widget creation, with 'w->core.num_popups' uninitialized. The
 *     workaround applied is to ignore the popup list if the 'num_popups'
 *     value is more than 50 or negative.
 *
 * Revision 1.1.1.1  2001/02/10 13:42:08  motiftools
 * Initial import of Xmt310 to CVS
 *
 *
 */

#ifndef _XmtCreate_h
#define _XmtCreate_h

/*
 * Names and classes for the pseudo-resources we read
 * during automatic widget creation.
 */
#define XmtNxmtChildren		"xmtChildren"
#define XmtCXmtChildren		"XmtChildren"
#define XmtNxmtType		"xmtType"
#define XmtCXmtType		"XmtType"
#define XmtNxmtCreationCallback	"xmtCreationCallback"
#define XmtCXmtCreationCallback "XmtCreationCallback"
#define XmtNxmtChildrenCreationCallback	"xmtChildrenCreationCallback"
#define XmtCXmtChildrenCreationCallback "XmtChildrenCreationCallback"
#define XmtNxmtManagedCreationCallback	"xmtManagedCreationCallback"
#define XmtCXmtManagedCreationCallback	"XmtManagedCreationCallback"


typedef struct {
    XrmQuark nameq;
    Widget *widgetp;
} XmtWidgetList;
    
_XFUNCPROTOBEGIN
#if NeedFunctionPrototypes
extern void XmtCreateChildren(Widget);
extern Widget XmtCreateChild(Widget, StringConst);
extern Widget XmtBuildDialog(Widget, StringConst, XtResourceList, Cardinal);  
extern Widget XmtBuildToplevel(Widget, StringConst);
extern Widget XmtBuildApplication(StringConst, StringConst, Display *,
				       ArgList, Cardinal);
extern void XmtCreateQueryListChildren(Widget, XmtWidgetList *, Cardinal);
extern Widget XmtCreateQueryListChild(Widget, StringConst, XmtWidgetList *,
				      Cardinal);
#else
extern void XmtCreateChildren();
extern Widget XmtCreateChild();
extern Widget XmtBuildDialog();  
extern Widget XmtBuildToplevel();
extern Widget XmtBuildApplication();
extern void XmtCreateQueryListChildren();
extern Widget XmtCreateQueryListChild();
#endif

#if NeedVarargsPrototypes
extern void XmtCreateQueryChildren(Widget, ...);
extern Widget XmtCreateQueryChild(Widget, StringConst, ...);
extern Widget XmtBuildQueryDialog(Widget, StringConst, XtResourceList,
				  Cardinal, ...);
extern Widget XmtBuildQueryToplevel(Widget, StringConst, ...);
extern Widget XmtBuildQueryApplication(StringConst, StringConst, Display *,
				       ArgList, Cardinal, ...);
#else
extern void XmtCreateQueryChildren();
extern Widget XmtCreateQueryChild();
extern Widget XmtBuildQueryDialog();
extern Widget XmtBuildQueryToplevel();
extern Widget XmtBuildQueryApplication();
#endif
_XFUNCPROTOEND    

#endif
