/* 
 * Motif Tools Library, Version 3.1
 * $Id: InputFieldP.h,v 1.3 2001/07/20 23:07:49 grmcdorman Exp $
 * 
 * Written by David Flanagan.
 * Copyright (c) 1992-2001 by David Flanagan.
 * All Rights Reserved.  See the file COPYRIGHT for details.
 * This is open source software.  See the file LICENSE for details.
 * There is no warranty for this software.  See NO_WARRANTY for details.
 *
 * $Log: InputFieldP.h,v $
 * Revision 1.3  2001/07/20 23:07:49  grmcdorman
 * This change provides two optional Motif/NuTCracker bug workarounds.
 * Defines in Xmt.tmpl (normally commented out) turn them on.
 *
 * Revision 1.2  2001/04/15 15:29:36  grmcdorman
 *   * XmtCallCallback has two new arguments types: XmtRCallbackUserData, which
 *     is the XmNuserData value, and XmtRCallbackImmediate, which is a value
 *     embedded in the list.
 *
 *   * Two new functions, XmtCreateQueryListChildren and XmtCreateQueryListChild,
 *     are provided. They are like the XmtCreate* functions except that they
 *     return a list of the widgets created (quark and pointer to the widget).
 *
 *   * Major enhancements to XmtInputField:
 *     - The input field now has a cancel() action. This reverts the field's
 *       value to that prior to the start of editing (i.e. the last commit).
 *       No default translation is bound to this action, however.
 *     - There is a callback for the cancel action (XmtNcancelCallback).
 *     - The input field can be set to cancel, commit, or do nothing on focus
 *       out. (Previously, it always did a commit). The default is commit.
 *       (XmtNfocusOutAction)
 *
 *   * XmtLayout:
 *     - It now has a losingFocusCallback (XmNloosingFocusCallback).
 *     - It, and the layout widgets (XmtLayoutBox/XmtLayoutString), now
 *       support background pixmaps.
 *     - It now supports XmNresizePolicy.
 *     - Strings in the layout parser are converted to XmStrings by
 *       XtConvertAndStore, so any custom XmString conversion is used
 *       (as, for example, the Xmt convert, which supports the @[]
 *       embedded items)
 *
 *   * Other:
 *     - Pixmaps are convered using XtConvertAndStore, again to allow
 *       custom converters.
 *
 * Bug Fixes: (for Solaris 2.5.1/Motif 1.2/Sparc system)
 *   * XmtInputField did not work properly if the widget had traversalOn
 *     false.
 *   * XmtLayout's child geometry manager was called by Xt when
 *     a shell child was added to the layout; it did not handle
 *     this case.
 *   * XmtLayout had an XmtWarningMsg call with a missing argument (line 2029).
 *   * Widgets automatically created by XmtLayout parsing have been given
 *     non-blank names. [You may wish to tweak this by adding, for example,
 *     a leading underscore.]
 *   * Under some circumstances, XmtNameToWidget could be called during
 *     widget creation, with 'w->core.num_popups' uninitialized. The
 *     workaround applied is to ignore the popup list if the 'num_popups'
 *     value is more than 50 or negative.
 *
 * Revision 1.1.1.1  2001/02/10 13:44:34  motiftools
 * Initial import of Xmt310 to CVS
 *
 *
 */

#ifndef _XmtInputFieldP_h
#define _XmtInputFieldP_h

#include <Xmt/XmtP.h>
#include <Xm/TextP.h>
#include <Xmt/InputField.h>
#include <Xmt/Symbols.h>

typedef struct _XmtInputFieldClassPart {
    XtPointer extension;
    XtActionProc    superclassGrabFocus;
} XmtInputFieldClassPart;

typedef struct _XmtInputFieldClassRec {
    CoreClassPart core_class;
    XmPrimitiveClassPart primitive_class;
    XmTextClassPart text_class;
    XmtInputFieldClassPart input_field_class;
} XmtInputFieldClassRec;

externalref XmtInputFieldClassRec xmtInputFieldClassRec;

typedef struct _XmtInputFieldPart {
    /* resources */
    String input;               /* The string that goes in the text widget */
    String pattern;             /* allowed characters for the string */
    Boolean overstrike;         /* overstrike mode? */
    String buffer_symbol_name;  /* the symbol name for the text value */
    String target_symbol_name;  /* the symbol name for the converted value */
    Boolean auto_insert;        /* auto insert constant chars in pattern? */
    Boolean auto_delete;        /* auto delete constant chars in pattern? */
    Boolean match_all;          /* input must match all of pattern? */
    XtCallbackList cancel_callback; /* called on Cancel action */
    XtCallbackList input_callback;  /* called on activate or focus out */
    XtCallbackList verify_callback; /* called before stuffing into buffer */
    XtCallbackList error_callback;  /* called when new value is bad */
    Boolean beep_on_error;      /* beep on bad input? */
    Boolean replace_on_error;   /*erase bad input and replace with old value?*/
    Boolean highlight_on_error; /* highlight bad input? */
    String error_string;        /* string to display on bad input */
    Pixel error_foreground;      /* color for highlighting bad input */
    Pixel error_background;      /* color for highlighting bad input */
    int focus_out_action;       /* Commit, cancel, or neither on focus out */
    /* internal state */
    Boolean input_changed;  /* whether value has changed since last stored */
    String error_value;     /* temp. storage for a bad value */
    short pattern_length;   /* strlen(pattern); */
    short pattern_skip;     /* state variable for HandlePattern() */
    Boolean error_state;    /* state variable for HandleError() & UndoError()*/
    XmtSymbol buffer_symbol;/* bound buffer symbol */
    XmtSymbol target_symbol;/* bound target symbol */
    Boolean ignore_symbol_notify;  /* prevent recursive loop */
    int max_length;         /* used with overstrike mode */
    Pixel foreground, background;  /* save old color values */
#ifdef NUTCRACKER_BUG_WORKAROUND
    Pixel sensitive_background;     /* NuTCracker WINTIF bug work-around */
#endif
    Boolean notTraversable; /* whether XmPrimitive traversalOn is False */
} XmtInputFieldPart;

typedef struct _XmtInputFieldRec {
    CorePart core;
    XmPrimitivePart primitive;
    XmTextPart text;
    XmtInputFieldPart input_field;
} XmtInputFieldRec;

#endif /* _XmtInputFieldP_h */
